package bookExamples.ch26Graphics.draw2d;

// Dave Pavlik,
// Created: 3/14/05
// Last modified: 3/16/05

import gui.run.RunButton;
import gui.run.RunSlider;
import gui.In;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.AffineTransform;
import java.beans.PropertyEditor;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

public class Pentagon extends DJShape implements PropertyEditor {
    String value = "Pentagon";
    int radius;
    double rotation = 0;
    double xScale = 1;
    double yScale = 1;
    double xShear = 0;
    double yShear = 0;
    PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    boolean transformAboutCenter = true;
    int selectedPointX = x1;
    int selectedPointY = y1;
    double transformedPointX = x1;
    double transformedPointY = y1;
    RunButton rb2;
    RunButton rb3;
    RunSlider rs1;
    RunSlider rs2;
    RunSlider rs3;
    RunSlider rs5;
    RunSlider rs6;
    RunSlider rs7;
    RunSlider rs8;
    private AffineTransform lastTransform = new AffineTransform();
    private AffineTransform priorTransform = new AffineTransform();

    public Pentagon(int x1, int y1, int x2, int y2) {
        super(x1, y1, x2, y2);
        this.x1 = x1;
        this.y1 = y1;
        int dx = x1 - x2;
        int dy = y1 - y2;
        radius = (int) Math.sqrt(dx * dx + dy * dy);
    }

    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new Pentagon(x1, y1, x2, y2);
    }

    public Point getCenter() {
        Point2D originalCenter = new Point2D.Double(x1, y1);
        Point2D transformedCenter = lastTransform.transform(originalCenter, null);

        return new Point((int) transformedCenter.getX(), (int) transformedCenter.getY());
    }

    public void setValue(Object value) {
        this.value = (String) value;
    }

    public Object getValue() {
        return value;
    }

    public boolean isPaintable() {
        return false;
    }

    public void paintValue(Graphics gfx, Rectangle box) {
        gfx.setClip(box);
        gfx.drawString("Click to edit...", box.x + 5, box.y + 15);
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String getAsText() {
        return null;
    }

    public void setAsText(String text) throws java.lang.IllegalArgumentException {
    }

    public String[] getTags() {
        return null;
    }

    public Component getCustomEditor() {
        JPanel jp = new JPanel(new GridLayout(0, 2, 2, 2));

        rs1 = new RunSlider(RunSlider.HORIZONTAL, 0, 1000, x1) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                selectedPointX = getValue();
            }
        };

        rs2 = new RunSlider(RunSlider.HORIZONTAL, 0, 1000, y1) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                selectedPointY = getValue();
            }
        };
        rs2.setOrientation(JSlider.VERTICAL);
        rs2.setInverted(true);

        jp.add(new JLabel("Color (click on color to select):"));
        RunButton rb1 = new RunButton() {
            public void run() {
                Color c = In.getColor();
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.setForeground(c);
                setBackground(c);
            }
        };
        rb1.setBorder(BorderFactory.createRaisedBevelBorder());
        rb1.setBackground(getForeground());
        jp.add(rb1);

        rb2 = new RunButton("Transform About Center") {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                transformAboutCenter = true;
                setBorder(BorderFactory.createLoweredBevelBorder());
                rb3.setBorder(BorderFactory.createRaisedBevelBorder());

                priorTransform = lastTransform;
                Point2D oldPoint = new Point2D.Double(x1, y1);
                Point2D newPoint = lastTransform.transform(oldPoint, null);
                transformedPointX = newPoint.getX();
                transformedPointY = newPoint.getY();
                rs1.setValue((int) transformedPointX);
                rs2.setValue((int) transformedPointY);

                setSlidersToZero();
            }
        };
        rb2.setBorder(BorderFactory.createLoweredBevelBorder());
        jp.add(rb2);

        rb3 = new RunButton("Transform About Selected Point") {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);

                transformAboutCenter = false;
                selectedPointX = (int) transformedPointX;
                selectedPointY = (int) transformedPointY;
                setBorder(BorderFactory.createLoweredBevelBorder());
                rb2.setBorder(BorderFactory.createRaisedBevelBorder());

                final JDialog jd = new JDialog(new JFrame(),
                        "Set Transform Point Location:", true);
                jd.setSize(200, 200);
                Container c = jd.getContentPane();
                c.setLayout(new BorderLayout());
                c.add(rs1, BorderLayout.NORTH);
                c.add(rs2, BorderLayout.EAST);
                c.add(new RunButton("OK") {
                    public void run() {
                        priorTransform = lastTransform;
                        transformedPointX = selectedPointX;
                        transformedPointY = selectedPointY;

                        setSlidersToZero();
                        jd.dispose();
                    }
                }, BorderLayout.CENTER);
                jd.show();
            }
        };
        rb3.setBorder(BorderFactory.createRaisedBevelBorder());
        jp.add(rb3);

        rs3 = new RunSlider(RunSlider.HORIZONTAL, 0, 360,
                (int) rotation) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.rotation = getValue();
            }
        };
        rs3.setBorder(BorderFactory.createTitledBorder("Rotation (degrees):"));
        jp.add(rs3);

        RunSlider rs4 = new RunSlider(RunSlider.HORIZONTAL, 0, 400, radius) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.radius = getValue();
            }
        };
        rs4.setBorder(BorderFactory.createTitledBorder("Radius (x 100):"));
        jp.add(rs4);

        rs5 = new RunSlider(RunSlider.HORIZONTAL, 0, 400,
                (int) xScale * 100) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.xScale = getValue() / 100f;
            }
        };
        rs5.setBorder(BorderFactory.createTitledBorder("Horizontal Scale (x 100):"));
        jp.add(rs5);

        rs6 = new RunSlider(RunSlider.HORIZONTAL, 0, 400,
                (int) yScale * 100) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.yScale = getValue() / 100f;
            }
        };
        rs6.setBorder(BorderFactory.createTitledBorder("Vertical Scale (x 100):"));
        jp.add(rs6);

        rs7 = new RunSlider(RunSlider.HORIZONTAL, -200, 200,
                (int) xShear * 100) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.xShear = getValue() / 100f;
            }
        };
        rs7.setBorder(BorderFactory.createTitledBorder("Horizontal Shear (x 100):"));
        jp.add(rs7);

        rs8 = new RunSlider(RunSlider.HORIZONTAL, -200, 200,
                (int) yShear * 100) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                Pentagon.this.yShear = getValue() / 100f;
            }
        };
        rs8.setBorder(BorderFactory.createTitledBorder("Vertical Shear (x 100):"));
        jp.add(rs8);

        RunSlider rs9 = new RunSlider(RunSlider.HORIZONTAL, -1000, 1000, x1) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                int dx = x2 - x1;
                x1 = getValue();
                x2 = x1 + dx;

                if (transformAboutCenter) {
                    transformAboutCenter = false;
                    selectedPointX = (int) transformedPointX;
                    selectedPointY = (int) transformedPointY;
                    rs1.setValue(selectedPointX);
                    rb2.setBorder(BorderFactory.createRaisedBevelBorder());
                    rb3.setBorder(BorderFactory.createLoweredBevelBorder());
                }
            }
        };
        rs9.setBorder(BorderFactory.createTitledBorder("X Translation:"));
        jp.add(rs9);

        RunSlider rs10 = new RunSlider(RunSlider.HORIZONTAL, -1000, 1000, y1) {
            public void run() {
                pcs.firePropertyChange("Pentagon", this, null);
                int dy = y2 - y1;
                y1 = getValue();
                y2 = y1 + dy;

                if (transformAboutCenter) {
                    transformAboutCenter = false;
                    selectedPointX = (int) transformedPointX;
                    selectedPointY = (int) transformedPointY;
                    rs2.setValue(selectedPointY);
                    rb2.setBorder(BorderFactory.createRaisedBevelBorder());
                    rb3.setBorder(BorderFactory.createLoweredBevelBorder());
                }
            }
        };
        rs10.setBorder(BorderFactory.createTitledBorder("Y Translation:"));
        jp.add(rs10);

        return jp;
    }

    private void setSlidersToZero() {
        rs3.setValue(0);
        rs5.setValue(100);
        rs6.setValue(100);
        rs7.setValue(0);
        rs8.setValue(0);
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        pcs.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        pcs.removePropertyChangeListener(listener);
    }

    public AffineTransform getTransform() {
        AffineTransform at1 = new AffineTransform();
        AffineTransform at2 = new AffineTransform();
        AffineTransform at3 = new AffineTransform();
        AffineTransform at4 = new AffineTransform();
        AffineTransform at5 = new AffineTransform();

        at1.translate(-transformedPointX, -transformedPointY);
        at2.rotate(Math.PI * rotation / 180);
        at3.scale(xScale, yScale);
        at4.shear(xShear, yShear);
        at5.translate(transformedPointX, transformedPointY);

        at1.concatenate(priorTransform);
        at2.concatenate(at1);
        at3.concatenate(at2);
        at4.concatenate(at3);
        at5.concatenate(at4);
        lastTransform = at5;

        return at5;
/*
        AffineTransform at = new AffineTransform();

        at.concatenate(priorTransform);
        at.translate(transformedPointX, transformedPointY);
        at.rotate(Math.PI * rotation / 180);
        at.scale(xScale, yScale);
        at.shear(xShear, yShear);
        at.translate(-transformedPointX, -transformedPointY);
        lastTransform = at;

        return at;
*/
    }


    public void draw(Graphics g) {
        CircleFcn circle = new CircleFcn(x1, y1, radius);
        double initialRotation = .75;
        GeneralPath gp = new GeneralPath();

        for (double t = 0; t < 1; t = t + 0.2) {
            Point2D point = circle.getPoint(t + initialRotation);
            if (t == 0) gp.moveTo((float) point.getX(), (float) point.getY());
            if (t > 0) gp.lineTo((float) point.getX(), (float) point.getY());
        }
        gp.closePath();

        Graphics2D g2d = (Graphics2D) g;
        Color saveColor = g2d.getColor();
        g2d.setColor(getForeground());

        AffineTransform at = getTransform();
        g2d.draw(gp.createTransformedShape(at));
        Point2D newCenter = at.transform(new Point(x1, y1), null);
        int newX1 = (int) newCenter.getX();
        int newY1 = (int) newCenter.getY();
        g2d.fillOval(newX1, newY1, 2, 2);
        g2d.drawString("(" + newX1 + ", " + newY1 + ")", newX1 + 3, newY1 + 3);

        if (!transformAboutCenter) { // draws cross-hairs on selected point
            g2d.drawLine(selectedPointX - 10, selectedPointY, selectedPointX + 10,
                    selectedPointY);
            g2d.drawLine(selectedPointX, selectedPointY - 10, selectedPointX,
                    selectedPointY + 10);
        }

        g2d.setColor(saveColor);
        restoreState(g2d);
    }
}

