/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package gui.layouts;

import java.awt.*;

/**
 * Copyright DocJava, inc. User: lyon
 * <p/>
 * Date: Mar 17, 2005
 * <p/>
 * Time: 4:24:00 PM
 */
public class VerticalFlowLayout implements LayoutManager {
    private int vgap = 0;
    private int hgap = 5;

    /**
     * VerticalFlowLayout constructor comment.
     */
    public VerticalFlowLayout() {
    }

    /**
     * VerticalFlowLayout constructor comment.
     */
    public VerticalFlowLayout(int vgap, int hgap) {
        if (vgap < 0) {
            this.vgap = 0;
        } else {
            this.vgap = vgap;
        }
        if (hgap < 0) {
            this.hgap = 0;
        } else {
            this.hgap = hgap;
        }
    }

    /**
     * addLayoutComponent method comment.
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * layoutContainer method comment.
     */
    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        int w = parent.getSize().width - insets.left - insets.right;
        int h = parent.getSize().height - insets.top - insets.bottom;
        int numComponents = parent.getComponentCount();
        if (numComponents == 0) {
            return;
        }
        int y = insets.top;
        int x = insets.left;
        int hmax = 0;
        for (int i = 0; i < numComponents; ++i) {
            Component c = parent.getComponent(i);
            if (c.isVisible()) {
                Dimension d = c.getMinimumSize();
                c.setBounds(x, y, d.width, d.height);
                y += d.height + vgap;
                if (d.width > hmax) hmax = d.width;
                if (y+d.height > h) {
                    y = vgap;
                    x = x + hgap + hmax;
                    hmax = 0;
                }
            }
        }
    }

    /**
     * minimumLayoutSize method comment.
     */
    public Dimension minimumLayoutSize(Container parent) {
        layoutContainer(parent);
        int w = 0;
        int h = 0;
        int numComponents = parent.getComponentCount();
        if (numComponents == 0) {
            return new Dimension(0,0);
        }
        for (int i = 0; i < numComponents; ++i) {
            Component c = parent.getComponent(i);
            if (c.isVisible()) {
                Dimension d = c.getMinimumSize();

                h += d.height + vgap;
                w += d.width + hgap;
            }
        }
        return new Dimension(w,h);
    }

    public Dimension preferredLayoutSize(Container parent) {
        return minimumLayoutSize(parent);
    }

    public void removeLayoutComponent(Component comp) {
    }

}
