package math.transforms;


public class IntegerDFT {

    private double r_data[] = null;
    private double i_data[] = null;
    private final static int scaleFactor = 100;
    private final static int PI = scale(3.1415926535);

    public IntegerDFT(int N) {
        r_data = new double[N];
        i_data = new double[N];
    }

    public IntegerDFT() {
    }


    public void graphs(String t) {
        setTitle(t);
    }

    public void setTitle(String t) {
    }



    public static double getMaxValue(double in[]) {
        double max;
        max = -0.99e30;
        for (int i = 0; i < in.length; i++)
            if (in[i] > max)
                max = in[i];
        return max;

    }


    private double sin(double d) {
        double v = Math.sin(d / scaleFactor);
        return scale(v);
    }

    private double cos(double d) {
        double v = Math.cos(d / (scaleFactor));
        return scale(v);
    }

    private final static int scale(double v) {
        return (int) (v * scaleFactor);
    }

    // todo, decide if you need this function.
    // if you have a fixed input, then you know how
    // many bits you need apriori...i.e. 256 samples
    // returns 8 as an answer.
    public static int log2(int n) {
        return (int) (Math.log(n) / Math.log(2.0));
    }

    public static int[] arrayCopy(int[] in) {
        int N = in.length;
        int out[] = new int[N];
        for (int i = 0; i < N; i++) {
            out[i] = in[i];
        }
        return out;
    }


    private void normalizeData() {
        int N = r_data.length;
        for (int k = 0; k < N; k++) {
            r_data[k] /= N;
            i_data[k] /= N;
        }

    }

    // assume that r_data and i_data are
    // set. Also assume that the real
    // value is to be returned
    public double[] idft() {
        int N = r_data.length;
        int twoPiOnN = 2 * PI / N;

        int twoPikOnN;
        int twoPijkOnN;

        double v[] = new double[N];

        System.out.println("Executing IDFT on " + N + " points...");


        for (int k = 0; k < N; k++) {
            twoPikOnN = twoPiOnN * k;
            for (int j = 0; j < N; j++) {
                twoPijkOnN = twoPikOnN * j;
                v[k] += r_data[j] * cos(twoPijkOnN)
                        - i_data[j] * sin(twoPijkOnN);

            }
        }
        return v;
    }

    public static void printArray(int[] v, String title) {
        System.out.println(title);
        for (int i = 0; i < v.length; i++) {
            System.out.println("v[" + i + "]=" + v[i]);
        }
    }


    public void printArrays(String title) {
        System.out.println(title);
        for (int i = 0; i < r_data.length; i++) {
            System.out.println("[" + i + "]=("
                    + r_data[i] + "," + i_data[i] + ")");
        }

    }

    public void printReal(String title) {
        System.out.println(title);
        for (int i = 0; i < r_data.length; i++) {
            System.out.println("r_data[" + i + "]="
                    + r_data[i]);
        }

    }


    public static void print(double o[]) {
        for (int i = 0; i < o.length; i++)
            System.out.println(o[i]);
    }

    public static void testDFT() {

        int N = 8;
        IntegerDFT f = new IntegerDFT(N);

        double v[];
        int x1[] = new int[N];
        for (int j = 0; j < N; j++)
            x1[j] = j;

        // take dft
        f.dft(x1);
        v = f.idft();
        System.out.println("j\tx1[j]\tre[j]\tim[j]\t v[j]");
        for (int j = 0; j < N; j++)
            System.out.println(j + "\t" +
                    x1[j] + "\t" +
                    f.r_data[j] + "\t" +
                    f.i_data[j] + "\t" +
                    Math.round(v[j]/(scaleFactor*scaleFactor)));

    }

    public static void main(String args[]) {
        //Oscillator.testAudioDft();  // this works!
        testDFT();

    }

    public void dft(int v[]) {
        int N = v.length;

        System.out.println("N=" + N);
        int twoPikOnN;
        int twoPijkOnN;

        // how many bits do we need?
        N = log2(N);
        //Truncate input data to a power of two
        // length = 2**(number of bits).
        N = 1 << N;
        System.out.println("PI=" + PI);
        int twoPiOnN = 2 * PI / N;
        System.out.println("twoPiOnN=" + twoPiOnN);
        // We truncate to a power of two so that
        // we can compare execution times with the FFT.
        // DFT generally does not need to truncate its input.

        r_data = new double[N];
        i_data = new double[N];


        System.out.println("Executing DFT on " + N + " points...");

        for (int k = 0; k < N; k++) {

            twoPikOnN = twoPiOnN * k;


            for (int j = 0; j < N; j++) {
                twoPijkOnN = twoPikOnN * j;
                r_data[k] += v[j] * cos(twoPijkOnN);
                i_data[k] -= v[j] * sin(twoPijkOnN);
            }
        }
        normalizeData();
    }


}
